@@ -107,7 +107,7 @@ class LogicGroup(LogicNode):
         for node in [self.left, self.right]:
             if isinstance(node, LogicGroup):
                 node.parent = self
-            
+
     def __repr__(self):
         """
         Provide a sensible representation of a node
@@ -125,6 +125,7 @@ class LogicGroup(LogicNode):
             return '(' + core + ')' 
         else:
             return core
+
     def get_codes(self):
         """
         Get a list of all constraint codes used in this group.
@@ -143,6 +144,12 @@ class LogicParseError(ReadableException):
     """
     pass
 
+class EmptyLogicError(ValueError):
+    """
+    An error representing the fact that an the logic string to be parsed was empty
+    """
+    pass
+
 class LogicParser(object):
     """
     Parses logic strings into logic groups
@@ -233,7 +240,7 @@ class LogicParser(object):
 
         @raise LogicParseError: if there is a syntax error in the logic
         """
-        def flatten(l): 
+        def flatten(l):
             """Flatten out a list which contains both values and sublists"""
             ret = []
             for item in l:
@@ -254,7 +261,9 @@ class LogicParser(object):
                 return x
 
         logic_str = logic_str.upper()
-        tokens = re.split("\s+", logic_str)
+        tokens = [t for t in re.split("\s+", logic_str) if t]
+        if not tokens:
+            raise EmptyLogicError()
         tokens = flatten([canonical(x, self.ops) for x in tokens])
         tokens = flatten([dedouble(x) for x in tokens])
         self.check_syntax(tokens)
@@ -373,18 +382,22 @@ class LogicParser(object):
         @raise AssertionError: is the tree doesn't have a unique root.
         """
         stack = []
-        for token in postfix_tokens:
-            if token not in self.ops:
-                stack.append(token)
-            else:
-                op = token
-                right = stack.pop()
-                left = stack.pop()
-                if not isinstance(right, LogicGroup): right = self.get_constraint(right)
-                if not isinstance(left, LogicGroup): left = self.get_constraint(left)
-                stack.append(LogicGroup(left, op, right))
-        assert len(stack) == 1, "Tree doesn't have a unique root"
-        return stack.pop()
+        try:
+            for token in postfix_tokens:
+                if token not in self.ops:
+                    stack.append(self.get_constraint(token))
+                else:
+                    op = token
+                    right = stack.pop()
+                    left = stack.pop()
+                    stack.append(LogicGroup(left, op, right))
+            assert len(stack) == 1, "Tree doesn't have a unique root"
+            return stack.pop()
+        except IndexError:
+            raise EmptyLogicError()
+
+class EmptyLogicError (IndexError):
+    pass
 
 class CodedConstraint(Constraint, LogicNode):
     """
@@ -419,6 +432,9 @@ class CodedConstraint(Constraint, LogicNode):
         self.code = code
         super(CodedConstraint, self).__init__(path)
 
+    def get_codes(self):
+        return [self.code]
+
     def __str__(self):
         """
         Stringify to the code they are refered to by.
