@@ -4,7 +4,7 @@ from xml.dom import minidom, getDOMImplementation
 
 from intermine.util import openAnything, ReadableException
 from intermine.pathfeatures import PathDescription, Join, SortOrder, SortOrderList
-from intermine.model import Column, Class, Model
+from intermine.model import Column, Class, Model, Reference
 import constraints
 
 """
@@ -332,6 +332,8 @@ class Query(object):
         self.name = ''
         self.description = ''
         self.service = service
+        self.prefetch_depth = service.prefetch_depth if service is not None  else 1
+        self.prefetch_id_only = service.prefetch_id_only if service is not None else False
         self.do_verification = validate
         self.path_descriptions = []
         self.joins = []
@@ -623,8 +625,10 @@ class Query(object):
                     views.append(str(p))
                 else:
                     views.append(str(p) + ".*")
+            elif isinstance(p, Reference):
+                views.append(p.name + ".*")
             else:
-                views.extend(re.split("(?:,?\s+|,)", p))
+                views.extend(re.split("(?:,?\s+|,)", str(p)))
 
         views = map(self.prefix_path, views)
 
@@ -632,10 +636,25 @@ class Query(object):
         for view in views:
             if view.endswith(".*"):
                 view = re.sub("\.\*$", "", view)
-                path = self.model.make_path(view, self.get_subclass_dict())
-                cd = path.end_class
-                attr_views = map(lambda x: view + "." + x.name, cd.attributes)
-                views_to_add.extend(attr_views)
+                scd = self.get_subclass_dict()
+                def expand(p, level, id_only=False):
+                    if level > 0:
+                        path = self.model.make_path(p, scd)
+                        cd = path.end_class
+                        add_f = lambda x: p + "." + x.name
+                        vs = [p + ".id"] if id_only and cd.has_id else map(add_f, cd.attributes)
+                        next_level = level - 1
+                        rs_and_cs = cd.references + cd.collections
+                        for r in rs_and_cs:
+                            rp = add_f(r)
+                            if next_level:
+                                self.outerjoin(rp)
+                            vs.extend(expand(rp, next_level, self.prefetch_id_only))
+                        return vs
+                    else:
+                        return []
+                depth = self.prefetch_depth
+                views_to_add.extend(expand(view, depth))
             else:
                 views_to_add.append(view)
 
@@ -720,7 +739,10 @@ class Query(object):
             if isinstance(args[0], tuple):
                 con = self.constraint_factory.make_constraint(*args[0])
             else:
-                con = args[0]
+                try:
+                    con = self.constraint_factory.make_constraint(*args[0].vargs, **args[0].kwargs)
+                except AttributeError:
+                    con = args[0]
         else:
             if len(args) == 0 and len(kwargs) == 1:
                 k, v = kwargs.items()[0]
@@ -743,28 +765,44 @@ class Query(object):
 
         return con
 
-    def where(self, *args, **kwargs):
+    def where(self, *cons, **kwargs):
         """
         Add a constraint to the query
         =============================
 
-        In contrast to add_constraint, this method returns self to support method chaining.
+        In contrast to add_constraint, this method returns
+        a new object with the given comstraint added.
 
         Also available as Query.filter
         """
-        self.add_constraint(*args, **kwargs)
-        return self
+        c = self.clone()
+        try:
+            for conset in cons:
+                codeds = c.coded_constraints
+                lstr = str(c.get_logic()) + " AND " if codeds else ""
+                start_c = chr(ord(codeds[-1].code) + 1) if codeds else 'A'
+                for con in conset:
+                    c.add_constraint(*con.vargs, **con.kwargs)
+                try:
+                    c.set_logic(lstr + conset.as_logic(start = start_c))
+                except constraints.EmptyLogicError:
+                    pass
+            for path, value in kwargs.items():
+                c.add_constraint(path, "=", value)
+        except AttributeError:
+            c.add_constraint(*cons, **kwargs)
+        return c
 
-    def column(self, string):
+    def column(self, col):
         """
         Return a Column object suitable for using to construct constraints with
         =======================================================================
 
-        This method is part of the SQLAlchemy compatible API.
+        This method is part of the SQLAlchemy style API.
 
         Also available as Query.c
         """
-        return self.model.column(self.prefix_path(string), self.get_subclass_dict(), self)
+        return self.model.column(self.prefix_path(str(col)), self.get_subclass_dict(), self)
 
     def verify_constraint_paths(self, cons=None):
         """
@@ -1013,7 +1051,13 @@ class Query(object):
         if isinstance(value, constraints.LogicGroup):
             logic = value
         else:
-            logic = self._logic_parser.parse(value)
+            try:
+                logic = self._logic_parser.parse(value)
+            except constraints.EmptyLogicError:
+                if self.coded_constraints:
+                    raise
+                else:
+                    return self
         if self.do_verification: self.validate_logic(logic)
         self._logic = logic
         return self
@@ -1049,6 +1093,11 @@ class Query(object):
         @rtype: L{intermine.pathfeatures.SortOrderList}
         """
         try:
+            v0 = self.views[0]
+            for j in self.joins:
+                if j.style == "OUTER":
+                    if v0.startswith(j.path):
+                        return ""
             return SortOrderList((self.views[0], SortOrder.ASC))
         except IndexError:
             raise QueryError("Query view is empty")
@@ -1235,6 +1284,16 @@ class Query(object):
         if len(to_run.views) == 0:
             to_run.add_view(to_run.root)
 
+        if "object" in row:
+            for c in self.coded_constraints:
+                p = to_run.column(c.path)._path
+                from_p = p if p.end_class is not None else p.prefix()
+                if not filter(lambda v: v.startswith(str(from_p)), to_run.views):
+                    if p.is_attribute():
+                        to_run.add_view(p)
+                    else:
+                        to_run.add_view(p.append("id"))
+
         path = to_run.get_results_path()
         params = to_run.to_query_params()
         params["start"] = start
