@@ -8,6 +8,7 @@ import httplib
 import re
 import copy
 import UserDict
+from itertools import groupby
 
 #class UJsonLibDecoder(object): # pragma: no cover
 #    def __init__(self):
@@ -29,7 +30,7 @@ except ImportError: # pragma: no cover
 
 # Local intermine imports
 from intermine.query import Query, Template
-from intermine.model import Model, Attribute, Reference, Collection
+from intermine.model import Model, Attribute, Reference, Collection, Column
 from intermine.util import ReadableException
 from intermine.lists.listmanager import ListManager
 
@@ -198,7 +199,9 @@ class Service(object):
     SCHEME                 = 'http://'
     SERVICE_RESOLUTION_PATH = "/check/"
 
-    def __init__(self, root, username=None, password=None, token=None):
+    def __init__(self, root,
+            username=None, password=None, token=None,
+            prefetch_depth=1, prefetch_id_only=False):
         """
         Constructor
         ===========
@@ -236,6 +239,8 @@ class Service(object):
         if not root.endswith("/service"): root = root + "/service"
 
         self.root = root
+        self.prefetch_depth = prefetch_depth
+        self.prefetch_id_only = prefetch_id_only
         self._templates = None
         self._model = None
         self._version = None
@@ -336,7 +341,7 @@ class Service(object):
             self._release = urllib.urlopen(self.root + self.RELEASE_PATH).read()
         return self._release
 
-    def new_query(self, root=None, xml=None):
+    def load_query(self, xml, root=None):
         """
         Construct a new Query object for the given webservice
         =====================================================
@@ -348,12 +353,9 @@ class Service(object):
 
         @return: L{intermine.query.Query}
         """
-        if xml is not None:
-            return Query.from_xml(xml, self.model, root=root)
-        else:
-            return Query(self.model, self, root=root)
+        return Query.from_xml(xml, self.model, root=root)
 
-    def select(self, *columns):
+    def select(self, *columns, **kwargs):
         """
         Construct a new Query object with the given columns selected.
         =============================================================
@@ -361,13 +363,22 @@ class Service(object):
         As new_query, except that instead of a root class, a list of
         output column expressions are passed instead.
         """
-        if len(columns) == 1 and not columns[0].endswith("*"):
+        if "xml" in kwargs:
+            return self.load_query(kwargs["xml"])
+        if len(columns) == 1:
             view = columns[0]
-            path = self.model.make_path(view)
-            if not path.is_attribute():
-                return Query(self.model, self).select(view + ".*")
+            if isinstance(view, Attribute):
+                return Query(self.model, self).select("%s.%s" % (view.declared_in.name, view))
+            if isinstance(view, Reference):
+                return Query(self.model, self).select("%s.%s.*" % (view.declared_in.name, view))
+            elif not isinstance(view, Column) and not str(view).endswith("*"):
+                path = self.model.make_path(view)
+                if not path.is_attribute():
+                    return Query(self.model, self).select(str(view) + ".*")
         return Query(self.model, self).select(*columns)
 
+    new_query = select
+
     def get_template(self, name):
         """
         Returns a template of the given name
@@ -499,11 +510,24 @@ class ResultObject(object):
 
     """
 
-    def __init__(self, data, cld):
+    def __init__(self, data, cld, view=[]):
+        stripped = [v[v.find(".") + 1:] for v in view]
+        self.selected_attributes = [v for v in stripped if "." not in v]
+        self.reference_paths = dict(((k, list(i)) for k, i in groupby(stripped, lambda x: x[:x.find(".") + 1])))
         self._data = data
-        self._cld = cld
+        self._cld = cld if "class" not in data or cld.name == data["class"] else cld.model.get_class(data["class"])
         self._attr_cache = {}
 
+    def __str__(self):
+        dont_show = set(["objectId", "class"])
+        return "%s(%s)" % (self._cld.name, ",  ".join("%s = %r" % (k, v) for k, v in self._data.items()
+            if not isinstance(v, dict) and not isinstance(v, list) and k not in dont_show))
+
+    def __repr__(self):
+        dont_show = set(["objectId", "class"])
+        return "%s(%s)" % (self._cld.name, ", ".join("%s = %r" % (k, getattr(self, k)) for k in self._data.keys()
+            if k not in dont_show))
+
     def __getattr__(self, name):
         if name in self._attr_cache:
             return self._attr_cache[name]
@@ -516,7 +540,10 @@ class ResultObject(object):
         if isinstance(fld, Attribute):
             if name in self._data:
                 attr = self._data[name]
+            if attr is None:
+                attr = self._fetch_attr(fld)
         elif isinstance(fld, Reference):
+            ref_paths = self._get_ref_paths(fld)
             if name in self._data:
                 data = self._data[name]
             else:
@@ -525,25 +552,47 @@ class ResultObject(object):
                 if data is None:
                     attr = []
                 else:
-                    attr = map(lambda x: ResultObject(x, fld.type_class), data)
+                    attr = map(lambda x: ResultObject(x, fld.type_class, ref_paths), data)
             else:
                 if data is None:
                     attr = None
                 else:
-                    attr = ResultObject(data, fld.type_class)
+                    attr = ResultObject(data, fld.type_class, ref_paths)
         else:
             raise WebserviceError("Inconsistent model - This should never happen")
         self._attr_cache[name] = attr
         return attr
 
-    def _fetch_reference(self, fld):
-        q = self._cld.model.service.new_query(self._cld.name)
-        q.select("id", fld.name + ".*")
-        q.where("id", "=", self._data["objectId"])
-        q.outerjoin(fld.name)
+    def _get_ref_paths(self, fld):
+        if fld.name + "." in self.reference_paths:
+            return self.reference_paths[fld.name + "."]
+        else:
+            return []
+
+    @property
+    def id(self):
+        """Return the internal DB identifier of this object. Or None if this is not an InterMine object"""
+        return self._data.get('objectId')
+
+    def _fetch_attr(self, fld):
+        if fld.name in self.selected_attributes:
+            return None # Was originally selected - no point asking twice
+        c = self._cld
+        if "id" not in c:
+            return None # Cannot reliably fetch anything without access to the objectId.
+        q = c.model.service.query(c, fld).where(id = self.id)
         r = q.first()
         return r._data[fld.name] if fld.name in r._data else None
 
+    def _fetch_reference(self, ref):
+        if ref.name + "." in self.reference_paths:
+            return None # Was originally selected - no point asking twice.
+        c = self._cld
+        if "id" not in c:
+            return None # Cannot reliably fetch anything without access to the objectId.
+        q = c.model.service.query(ref).outerjoin(ref).where(id = self.id)
+        r = q.first()
+        return r._data[ref.name] if ref.name in r._data else None
 
 class ResultRow(object):
     """
@@ -759,7 +808,7 @@ class ResultIterator(object):
                 "list"        : lambda: JSONIterator(con, lambda x: self.row(x, self.view).to_l()),
                 "rr"          : lambda: JSONIterator(con, lambda x: self.row(x, self.view)),
                 "dict"        : lambda: JSONIterator(con, lambda x: self.row(x, self.view).to_d()),
-                "jsonobjects" : lambda: JSONIterator(con, lambda x: ResultObject(x, self.cld))
+                "jsonobjects" : lambda: JSONIterator(con, lambda x: ResultObject(x, self.cld, self.view))
             }.get(self.rowformat)()
         except Exception, e:
             raise Exception("Couldn't get iterator for "  + self.rowformat + str(e))
