@@ -304,6 +304,127 @@ class TestQuery(WebserviceTest): # pragma: no cover
         except ConstraintError, ex:
             self.assertEqual(ex.message, "'Employee.department' does not represent an attribute")
 
+    def testIDOnlyExpansion(self):
+        """Should be able to expand to a given depth, selecting only the id attribute for IM Object classes"""
+        self.q.prefetch_depth = 3
+        self.q.prefetch_id_only = True
+
+        self.q.add_view("Employee.*")
+        expected = [
+            'Employee.age',
+            'Employee.end',
+            'Employee.fullTime',
+            'Employee.id',
+            'Employee.name',
+            'Employee.address.id',
+            'Employee.department.id',
+            'Employee.department.company.id',
+            'Employee.department.manager.id',
+            'Employee.department.employees.id',
+            'Employee.department.rejectedEmployees.id',
+            'Employee.departmentThatRejectedMe.id',
+            'Employee.departmentThatRejectedMe.company.id',
+            'Employee.departmentThatRejectedMe.manager.id',
+            'Employee.departmentThatRejectedMe.employees.id',
+            'Employee.departmentThatRejectedMe.rejectedEmployees.id',
+            'Employee.simpleObjects.name',
+            'Employee.simpleObjects.employee.id'
+        ]
+        self.assertEqual(self.q.views, expected)
+        exp_joins = ['Employee.address',
+            'Employee.department',
+            'Employee.department.company',
+            'Employee.department.manager',
+            'Employee.department.employees',
+            'Employee.department.rejectedEmployees',
+            'Employee.departmentThatRejectedMe',
+            'Employee.departmentThatRejectedMe.company',
+            'Employee.departmentThatRejectedMe.manager',
+            'Employee.departmentThatRejectedMe.employees',
+            'Employee.departmentThatRejectedMe.rejectedEmployees',
+            'Employee.simpleObjects',
+            'Employee.simpleObjects.employee']
+        self.assertEqual([j.path for j in self.q.joins], exp_joins)
+
+    def testDeepViews(self):
+        """Should be able to expand to a given prefetch depth"""
+        self.q.prefetch_depth = 3
+        self.q.add_view("Employee.*")
+        expected = [
+           'Employee.age',
+           'Employee.end',
+           'Employee.fullTime',
+           'Employee.id',
+           'Employee.name',
+           'Employee.address.address',
+           'Employee.address.id',
+           'Employee.department.id',
+           'Employee.department.name',
+           'Employee.department.company.id',
+           'Employee.department.company.name',
+           'Employee.department.company.vatNumber',
+           'Employee.department.manager.age',
+           'Employee.department.manager.end',
+           'Employee.department.manager.fullTime',
+           'Employee.department.manager.id',
+           'Employee.department.manager.name',
+           'Employee.department.manager.seniority',
+           'Employee.department.manager.title',
+           'Employee.department.employees.age',
+           'Employee.department.employees.end',
+           'Employee.department.employees.fullTime',
+           'Employee.department.employees.id',
+           'Employee.department.employees.name',
+           'Employee.department.rejectedEmployees.age',
+           'Employee.department.rejectedEmployees.end',
+           'Employee.department.rejectedEmployees.fullTime',
+           'Employee.department.rejectedEmployees.id',
+           'Employee.department.rejectedEmployees.name',
+           'Employee.departmentThatRejectedMe.id',
+           'Employee.departmentThatRejectedMe.name',
+           'Employee.departmentThatRejectedMe.company.id',
+           'Employee.departmentThatRejectedMe.company.name',
+           'Employee.departmentThatRejectedMe.company.vatNumber',
+           'Employee.departmentThatRejectedMe.manager.age',
+           'Employee.departmentThatRejectedMe.manager.end',
+           'Employee.departmentThatRejectedMe.manager.fullTime',
+           'Employee.departmentThatRejectedMe.manager.id',
+           'Employee.departmentThatRejectedMe.manager.name',
+           'Employee.departmentThatRejectedMe.manager.seniority',
+           'Employee.departmentThatRejectedMe.manager.title',
+           'Employee.departmentThatRejectedMe.employees.age',
+           'Employee.departmentThatRejectedMe.employees.end',
+           'Employee.departmentThatRejectedMe.employees.fullTime',
+           'Employee.departmentThatRejectedMe.employees.id',
+           'Employee.departmentThatRejectedMe.employees.name',
+           'Employee.departmentThatRejectedMe.rejectedEmployees.age',
+           'Employee.departmentThatRejectedMe.rejectedEmployees.end',
+           'Employee.departmentThatRejectedMe.rejectedEmployees.fullTime',
+           'Employee.departmentThatRejectedMe.rejectedEmployees.id',
+           'Employee.departmentThatRejectedMe.rejectedEmployees.name',
+           'Employee.simpleObjects.name',
+           'Employee.simpleObjects.employee.age',
+           'Employee.simpleObjects.employee.end',
+           'Employee.simpleObjects.employee.fullTime',
+           'Employee.simpleObjects.employee.id',
+           'Employee.simpleObjects.employee.name'
+           ]
+        self.assertEqual(self.q.views, expected)
+        exp_joins = ['Employee.address',
+            'Employee.department',
+            'Employee.department.company',
+            'Employee.department.manager',
+            'Employee.department.employees',
+            'Employee.department.rejectedEmployees',
+            'Employee.departmentThatRejectedMe',
+            'Employee.departmentThatRejectedMe.company',
+            'Employee.departmentThatRejectedMe.manager',
+            'Employee.departmentThatRejectedMe.employees',
+            'Employee.departmentThatRejectedMe.rejectedEmployees',
+            'Employee.simpleObjects',
+            'Employee.simpleObjects.employee']
+        self.assertEqual([j.path for j in self.q.joins], exp_joins)
+
     def testViewAlias(self):
         """The aliases for add_view should work as well"""
         self.q.select("Employee.age")
@@ -600,7 +721,7 @@ class TestQuery(WebserviceTest): # pragma: no cover
 
         # SQLAlchemy style
         q = self.service.query(Employee).\
-                add_columns("name", "age", "department.name").\
+                select("name", "age", "department.name").\
                 filter(Employee.name != None).\
                 filter(Employee.age > 10).\
                 filter(Employee.department % ("Sales", "Wernham-Hogg")).\
@@ -614,6 +735,49 @@ class TestQuery(WebserviceTest): # pragma: no cover
 
         self.assertEqual(expected, q.to_xml())
 
+    def testKWCons(self):
+        """Test use of constraints provided in kwargs"""
+
+        model = self.q.model
+
+        expected = '<query longDescription="" model="testmodel" name="" sortOrder="Employee.name asc" view="Employee.name"><constraint code="A" op="=" path="Employee.age" value="10"/></query>'
+
+        q = model.Employee.select("name").where(age = 10)
+
+        self.assertEqual(expected, q.to_xml())
+
+    def testLogicConstraintTrees(self):
+        # Actually SQL-Alchemy-esque
+        expected = """
+            <query constraintLogic="(A and B) or (C and D)" longDescription="" model="testmodel" name=""
+                sortOrder="Employee.age asc" view="Employee.age Employee.end Employee.fullTime
+                Employee.id Employee.name Employee.department.name">
+                <join path="Employee.department" style="OUTER"/>
+                <constraint code="A" op="IS NOT NULL" path="Employee.name"/>
+                <constraint code="B" op="&gt;" path="Employee.age" value="10"/>
+                <constraint code="C" op="IN" path="Employee" value="my-list"/>
+                <constraint code="D" op="LOOKUP" path="Employee.department.manager" value="David"/>
+                <constraint path="Employee.department.manager" type="CEO"/>
+            </query>
+            """
+        e = self.service.model.Employee
+        CEO = self.service.model.CEO
+        q = self.service.query(e, e.department.name).\
+                filter(
+                        e.department.manager < CEO,
+                        (
+                            ((e.name != None) & (e.age > 10))
+                            | (e.in_(self.l) & (e.department.manager % "David"))
+                        )
+                    ).\
+                outerjoin(e.department).\
+                order_by(e.age)
+
+        expected = re.sub(r'\s+', ' ', expected)
+        expected = re.sub(r'>\s+<', '><', expected)
+        expected = expected.strip()
+        self.assertEqual(expected, q.to_xml())
+
 class TestTemplate(TestQuery): # pragma: no cover
 
     expected_unary = '[<TemplateUnaryConstraint: Employee.age IS NULL (editable, locked)>, <TemplateUnaryConstraint: Employee.name IS NOT NULL (editable, locked)>]'
@@ -638,6 +802,8 @@ class TestQueryResults(WebserviceTest): # pragma: no cover
         QUERY_PATH = '/QUERY-PATH'
         TEMPLATEQUERY_PATH = '/TEMPLATE-PATH'
         root = 'ROOT'
+        prefetch_depth = 1
+        prefetch_id_only = False
 
         def get_results(self, *args):
             return args
